@@ -1,15 +1,8 @@
+// SPDX-FileCopyrightText: 2013 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+//
+// SPDX-License-Identifier: MIT
+
 /*
- * Copyright (C) 2013  Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
- *
- * THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY EXPRESSED
- * OR IMPLIED.  ANY USE IS AT YOUR OWN RISK.
- *
- * Permission is hereby granted to use or copy this program for any
- * purpose,  provided the above notices are retained on all copies.
- * Permission to modify the code and to distribute modified code is
- * granted, provided the above notices are retained, and a notice that
- * the code was modified is included with the above copyright notice.
- *
  * This example shows how to iterate on duplicate keys within a RCU
  * lock-free hash table. This hash table requires using a RCU scheme.
  */
@@ -42,7 +35,7 @@ int match(struct cds_lfht_node *ht_node, const void *_key)
 	return *key == node->value;
 }
 
-int main(int argc, char **argv)
+int main(void)
 {
 	int values[] = { -5, 42, 42, 36, 24, };	/* 42 is duplicated */
 	int lookup_values[] = { 42, 200, 36, };
